@@ -214,6 +214,7 @@ gjs_callback_closure(ffi_cif *cif,
 
     trampoline = data;
     g_assert(trampoline);
+    gjs_callback_trampoline_ref(trampoline);
 
     context = gjs_runtime_get_current_context(trampoline->runtime);
     JS_BeginRequest(context);
@@ -279,10 +280,13 @@ gjs_callback_closure(ffi_cif *cif,
         }
     }
 
-    if (trampoline->is_vfunc)
+    if (trampoline->is_vfunc) {
         this_object = JSVAL_TO_OBJECT(jsargs[0]);
-    else
+        jsargs++;
+        n_jsargs--;
+    } else {
         this_object = NULL;
+    }
 
     if (!JS_CallFunctionValue(context,
                               this_object,
@@ -406,6 +410,7 @@ out:
         completed_trampolines = g_slist_prepend(completed_trampolines, trampoline);
     }
 
+    gjs_callback_trampoline_unref(trampoline);
     JS_EndRequest(context);
 }
 
@@ -539,87 +544,6 @@ get_length_from_arg (GArgument *arg, GITypeTag tag)
     }
 }
 
-/* Extract the correct bits from an ffi_arg return value into
- * GIArgument: https://bugzilla.gnome.org/show_bug.cgi?id=665152
- *
- * Also see the ffi_call man page - the storage requirements for return
- * values are "special".
- */
-union GjsFFIReturnValue {
-    ffi_arg  rv_ffi_arg;
-    ffi_sarg rv_ffi_sarg;
-    guint64  rv_u64;
-    gint64   rv_s64;
-    float    rv_float;
-    double   rv_double;
-};
-static void
-set_gargument_from_ffi_return_value (GITypeInfo                  *return_info,
-                                     GIArgument                  *return_value,
-                                     union GjsFFIReturnValue     *value)
-{
-    switch (g_type_info_get_tag (return_info)) {
-    case GI_TYPE_TAG_INT8:
-        return_value->v_int8 = (gint8) value->rv_ffi_sarg;
-        break;
-    case GI_TYPE_TAG_UINT8:
-        return_value->v_uint8 = (guint8) value->rv_ffi_arg;
-        break;
-    case GI_TYPE_TAG_INT16:
-        return_value->v_int16 = (gint16) value->rv_ffi_sarg;
-        break;
-    case GI_TYPE_TAG_UINT16:
-        return_value->v_uint16 = (guint16) value->rv_ffi_arg;
-        break;
-    case GI_TYPE_TAG_INT32:
-        return_value->v_int32 = (gint32) value->rv_ffi_sarg;
-        break;
-    case GI_TYPE_TAG_UINT32:
-    case GI_TYPE_TAG_BOOLEAN:
-    case GI_TYPE_TAG_UNICHAR:
-        return_value->v_uint32 = (guint32) value->rv_ffi_arg;
-                        
-        break;
-    case GI_TYPE_TAG_INT64:
-        return_value->v_int64 = (gint64) value->rv_s64;
-        break;
-    case GI_TYPE_TAG_UINT64:
-        return_value->v_uint64 = (guint64) value->rv_u64;
-        break;
-    case GI_TYPE_TAG_FLOAT:
-        return_value->v_float = value->rv_float;
-        break;
-    case GI_TYPE_TAG_DOUBLE:
-        return_value->v_double = value->rv_double;
-        break;
-    case GI_TYPE_TAG_INTERFACE:
-        {
-            GIBaseInfo* interface_info;
-            GIInfoType interface_type;
-
-            interface_info = g_type_info_get_interface(return_info);
-            interface_type = g_base_info_get_type(interface_info);
-
-            switch(interface_type) {
-            case GI_INFO_TYPE_ENUM:
-            case GI_INFO_TYPE_FLAGS:
-                return_value->v_int32 = (gint32) value->rv_ffi_sarg;
-                break;
-            default:
-                return_value->v_pointer = (gpointer) value->rv_ffi_arg;
-                break;
-            }
-
-            g_base_info_unref(interface_info);
-        }
-        break;
-    default:
-        return_value->v_pointer = (gpointer) value->rv_ffi_arg;
-        break;
-    }
-}
-
-
 static JSBool
 gjs_invoke_c_function(JSContext      *context,
                       Function       *function,
@@ -643,7 +567,7 @@ gjs_invoke_c_function(JSContext      *context,
     GArgument *out_arg_cvalues;
     GArgument *inout_original_arg_cvalues;
     gpointer *ffi_arg_pointers;
-    union GjsFFIReturnValue return_value;
+    GIFFIReturnValue return_value;
     gpointer return_value_p; /* Will point inside the union return_value */
     GArgument return_gargument;
 
@@ -996,13 +920,13 @@ gjs_invoke_c_function(JSContext      *context,
 
     /* See comment for GjsFFIReturnValue above */
     if (return_tag == GI_TYPE_TAG_FLOAT)
-        return_value_p = &return_value.rv_float;
+        return_value_p = &return_value.v_float;
     else if (return_tag == GI_TYPE_TAG_DOUBLE)
-        return_value_p = &return_value.rv_double;
+        return_value_p = &return_value.v_double;
     else if (return_tag == GI_TYPE_TAG_INT64 || return_tag == GI_TYPE_TAG_UINT64)
-        return_value_p = &return_value.rv_u64;
+        return_value_p = &return_value.v_uint64;
     else
-        return_value_p = &return_value.rv_ffi_arg;
+        return_value_p = &return_value.v_long;
     ffi_call(&(function->invoker.cif), function->invoker.native_address, return_value_p, ffi_arg_pointers);
 
     gjs_runtime_pop_context(JS_GetRuntime(context));
@@ -1031,7 +955,7 @@ gjs_invoke_c_function(JSContext      *context,
 
             g_assert_cmpuint(next_rval, <, function->js_out_argc);
 
-            set_gargument_from_ffi_return_value(&return_info, &return_gargument, &return_value);
+            gi_type_info_extract_ffi_return_value(&return_info, &return_value, &return_gargument);
 
             array_length_pos = g_type_info_get_array_length(&return_info);
             if (array_length_pos >= 0) {
