@@ -36,16 +36,13 @@ JSBool    gjs_define_object_class       (JSContext     *context,
                                          JSObject      *in_object,
                                          GType          gtype,
                                          JSObject     **constructor_p,
-                                         JSObject     **prototype_p,
-                                         GIObjectInfo **class_info_p);
+                                         JSObject     **prototype_p);
 JSObject* gjs_lookup_object_prototype   (JSContext     *context,
                                          GType          gtype);
 JSObject* gjs_object_from_g_object      (JSContext     *context,
                                          GObject       *gobj);
 GObject*  gjs_g_object_from_object      (JSContext     *context,
                                          JSObject      *obj);
-GType     gjs_gtype_from_value         (JSContext     *context,
-                                        jsval          value);
 
 G_END_DECLS
 
