@@ -35,7 +35,7 @@
 #include <gjs/compat.h>
 #include "repo.h"
 #include "function.h"
-
+#include "gtype.h"
 
 #include <jsapi.h>
 
@@ -112,7 +112,9 @@ union_new_resolve(JSContext *context,
 
             union_proto = obj;
 
-            if (gjs_define_function(context, union_proto, method_info) == NULL) {
+            if (gjs_define_function(context, union_proto,
+                                    g_registered_type_info_get_g_type(priv->info),
+                                    method_info) == NULL) {
                 g_base_info_unref( (GIBaseInfo*) method_info);
                 ret = JS_FALSE;
                 goto out;
@@ -211,10 +213,6 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(union)
     proto = JS_GetPrototype(context, object);
     gjs_debug_lifecycle(GJS_DEBUG_GBOXED, "union instance __proto__ is %p", proto);
 
-    gjs_debug_lifecycle(GJS_DEBUG_GBOXED,
-                        "union instance constructing proto %d, obj class %s proto class %s",
-                        is_proto, obj_class->name, proto_class->name);
-
     /* If we're the prototype, then post-construct we'll fill in priv->info.
      * If we are not the prototype, though, then we'll get ->info from the
      * prototype and then create a GObject if we don't have one already.
@@ -382,6 +380,7 @@ gjs_define_union_class(JSContext    *context,
     jsval value;
     Union *priv;
     GType gtype;
+    JSObject *constructor;
 
     /* For certain unions, we may be able to relax this in the future by
      * directly allocating union memory, as we do for structures in boxed.c
@@ -400,8 +399,6 @@ gjs_define_union_class(JSContext    *context,
     constructor_name = g_base_info_get_name( (GIBaseInfo*) info);
 
     if (gjs_object_get_property(context, in_object, constructor_name, &value)) {
-        JSObject *constructor;
-
         if (!JSVAL_IS_OBJECT(value)) {
             gjs_throw(context, "Existing property '%s' does not look like a constructor",
                          constructor_name);
@@ -461,20 +458,24 @@ gjs_define_union_class(JSContext    *context,
     gjs_debug(GJS_DEBUG_GBOXED, "Defined class %s prototype is %p class %p in object %p",
               constructor_name, prototype, JS_GET_CLASS(context, prototype), in_object);
 
-    if (constructor_p) {
-        *constructor_p = NULL;
-        gjs_object_get_property(context, in_object, constructor_name, &value);
-        if (value != JSVAL_VOID) {
-            if (!JSVAL_IS_OBJECT(value)) {
-                gjs_throw(context, "Property '%s' does not look like a constructor",
-                          constructor_name);
-                return JS_FALSE;
-            }
+    gjs_object_get_property(context, in_object, constructor_name, &value);
+    if (!JSVAL_IS_VOID(value)) {
+        if (!JSVAL_IS_OBJECT(value)) {
+            gjs_throw(context, "Property '%s' does not look like a constructor",
+                      constructor_name);
+            return JS_FALSE;
         }
-
-        *constructor_p = JSVAL_TO_OBJECT(value);
     }
 
+    constructor = JSVAL_TO_OBJECT(value);
+    
+    value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, gtype));
+    JS_DefineProperty(context, constructor, "$gtype", value,
+                      NULL, NULL, JSPROP_PERMANENT);
+
+    if (constructor_p)
+        *constructor_p = constructor;
+
     if (prototype_p)
         *prototype_p = prototype;
 
