@@ -396,7 +396,7 @@ gjs_object_get_property(JSContext  *context,
 
     JS_EndRequest(context);
 
-    return value != JSVAL_VOID;
+    return !JSVAL_IS_VOID(value);
 }
 
 /* Returns whether the object had the property; if the object did
@@ -423,7 +423,7 @@ gjs_object_require_property(JSContext       *context,
     if (value_p)
         *value_p = value;
 
-    if (value != JSVAL_VOID) {
+    if (!JSVAL_IS_VOID(value)) {
         JS_ClearPendingException(context); /* in case JS_GetProperty() was on crack */
         JS_EndRequest(context);
         return TRUE;
@@ -539,7 +539,7 @@ gjs_init_class_dynamic(JSContext      *context,
                                          class_copy->base.name, &value))
             goto error;
     }
-    g_assert(value != JSVAL_VOID);
+    g_assert(!JSVAL_IS_VOID(value));
     g_assert(prototype != NULL);
 
     /* Now manually define our constructor with a sane name, in the
@@ -1257,7 +1257,7 @@ gjs_get_type_name(jsval value)
 {
     if (JSVAL_IS_NULL(value)) {
         return "null";
-    } else if (value == JSVAL_VOID) {
+    } else if (JSVAL_IS_VOID(value)) {
         return "undefined";
     } else if (JSVAL_IS_INT(value)) {
         return "integer";
@@ -1373,13 +1373,18 @@ gjs_value_to_int64  (JSContext  *context,
  * types of the expected arguments, and a list of argument names and
  * value location pairs.  The currently accepted format specifiers are:
  *
+ * b: A boolean
  * s: A string, converted into UTF-8
  * z: Like 's', but may be null in JavaScript (which appears as NULL in C)
  * F: A string, converted into "filename encoding" (i.e. active locale)
  * i: A number, will be converted to a C "gint32"
  * u: A number, converted into a C "guint32"
+ * t: A 64-bit number, converted into a C "gint64" by way of gjs_value_to_int64()
  * o: A JavaScript object, as a "JSObject *"
  *
+ * If the first character in the format string is a '!', then JS is allowed
+ * to pass extra arguments that are ignored, to the function.
+ *
  * The '|' character introduces optional arguments.  All format specifiers
  * after a '|' when not specified, do not cause any changes in the C
  * value location.
@@ -1399,6 +1404,7 @@ gjs_parse_args (JSContext  *context,
     guint n_unwind = 0;
 #define MAX_UNWIND_STRINGS 16
     gpointer unwind_strings[MAX_UNWIND_STRINGS];
+    gboolean ignore_trailing_args = FALSE;
     guint n_required;
     guint n_total;
     guint consumed_args;
@@ -1407,6 +1413,11 @@ gjs_parse_args (JSContext  *context,
 
     va_start (args, argv);
 
+    if (*format == '!') {
+        ignore_trailing_args = TRUE;
+        format++;
+    }
+
     /* Check for optional argument specifier */
     fmt_iter = strchr (format, '|');
     if (fmt_iter) {
@@ -1419,7 +1430,7 @@ gjs_parse_args (JSContext  *context,
         n_required = n_total = strlen (format);
     }
 
-    if (argc < n_required || argc > n_total) {
+    if (argc < n_required || (argc > n_total && !ignore_trailing_args)) {
         if (n_required == n_total) {
             gjs_throw(context, "Error invoking %s: Expected %d arguments, got %d", function_name,
                       n_required, argc);
@@ -1459,6 +1470,15 @@ gjs_parse_args (JSContext  *context,
         js_value = argv[consumed_args];
 
         switch (*fmt_iter) {
+        case 'b': {
+            if (!JSVAL_IS_BOOLEAN(js_value)) {
+                arg_error_message = "Not a boolean";
+            } else {
+                gboolean *arg = arg_location;
+                *arg = JSVAL_TO_BOOLEAN(js_value);
+            }
+        }
+            break;
         case 'o': {
             if (!JSVAL_IS_OBJECT(js_value)) {
                 arg_error_message = "Not an object";
@@ -1512,6 +1532,14 @@ gjs_parse_args (JSContext  *context,
             }
         }
             break;
+        case 't': {
+            if (!gjs_value_to_int64(context, js_value, (gint64*) arg_location)) {
+                /* Our error message is going to be more useful */
+                JS_ClearPendingException(context);
+                arg_error_message = "Couldn't convert to 64-bit integer";
+            }
+        }
+            break;
         case 'f': {
             double num;
             if (!JS_ValueToNumber(context, js_value, &num)) {
